@@ -1,9 +1,5 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from django.contrib.auth import validators
 from django.db import migrations, models
-from django.utils import six
 
 
 class Migration(migrations.Migration):
@@ -21,7 +17,7 @@ class Migration(migrations.Migration):
                 help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.',
                 max_length=150,
                 unique=True,
-                validators=[validators.UnicodeUsernameValidator() if six.PY3 else validators.ASCIIUsernameValidator()],
+                validators=[validators.UnicodeUsernameValidator()],
                 verbose_name='username',
             ),
         ),
