@@ -87,15 +87,21 @@ class GEOSGeometryBase(GEOSBase):
         return '<%s object at %s>' % (self.geom_type, hex(addressof(self.ptr)))
 
     # Pickling support
+    def _to_pickle_wkb(self):
+        return bytes(self.wkb)
+
+    def _from_pickle_wkb(self, wkb):
+        return wkb_r().read(memoryview(wkb))
+
     def __getstate__(self):
         # The pickled state is simply a tuple of the WKB (in string form)
         # and the SRID.
-        return bytes(self.wkb), self.srid
+        return self._to_pickle_wkb(), self.srid
 
     def __setstate__(self, state):
         # Instantiating from the tuple state that was pickled.
         wkb, srid = state
-        ptr = wkb_r().read(memoryview(wkb))
+        ptr = self._from_pickle_wkb(wkb)
         if not ptr:
             raise GEOSException('Invalid Geometry loaded from pickled state.')
         self.ptr = ptr
@@ -499,6 +505,18 @@ class GEOSGeometryBase(GEOSBase):
         """
         return self._topology(capi.geos_buffer(self.ptr, width, quadsegs))
 
+    def buffer_with_style(self, width, quadsegs=8, end_cap_style=1, join_style=1, mitre_limit=5.0):
+        """
+        Same as buffer() but allows customizing the style of the buffer.
+
+        End cap style can be round (1), flat (2), or square (3).
+        Join style can be round (1), mitre (2), or bevel (3).
+        Mitre ratio limit only affects mitered join style.
+        """
+        return self._topology(
+            capi.geos_bufferwithstyle(self.ptr, width, quadsegs, end_cap_style, join_style, mitre_limit),
+        )
+
     @property
     def centroid(self):
         """
