@@ -1,5 +1,5 @@
 import builtins
-import collections
+import collections.abc
 import datetime
 import decimal
 import enum
@@ -8,11 +8,12 @@ import math
 import re
 import types
 import uuid
+from collections import OrderedDict
 
+from django.conf import SettingsReference
 from django.db import models
 from django.db.migrations.operations.base import Operation
 from django.db.migrations.utils import COMPILED_REGEX_TYPE, RegexObject
-from django.utils import datetime_safe
 from django.utils.functional import LazyObject, Promise
 from django.utils.timezone import utc
 from django.utils.version import get_docs_version
@@ -46,30 +47,21 @@ class BaseSimpleSerializer(BaseSerializer):
         return repr(self.value), set()
 
 
-class ByteTypeSerializer(BaseSerializer):
+class DateTimeSerializer(BaseSerializer):
+    """For datetime.*, except datetime.datetime."""
     def serialize(self):
-        return repr(self.value), set()
+        return repr(self.value), {'import datetime'}
 
 
-class DatetimeSerializer(BaseSerializer):
+class DatetimeDatetimeSerializer(BaseSerializer):
+    """For datetime.datetime."""
     def serialize(self):
         if self.value.tzinfo is not None and self.value.tzinfo != utc:
             self.value = self.value.astimezone(utc)
-        value_repr = repr(self.value).replace("<UTC>", "utc")
-        if isinstance(self.value, datetime_safe.datetime):
-            value_repr = "datetime.%s" % value_repr
         imports = ["import datetime"]
         if self.value.tzinfo is not None:
             imports.append("from django.utils.timezone import utc")
-        return value_repr, set(imports)
-
-
-class DateSerializer(BaseSerializer):
-    def serialize(self):
-        value_repr = repr(self.value)
-        if isinstance(self.value, datetime_safe.date):
-            value_repr = "datetime.%s" % value_repr
-        return value_repr, {"import datetime"}
+        return repr(self.value).replace('<UTC>', 'utc'), set(imports)
 
 
 class DecimalSerializer(BaseSerializer):
@@ -124,9 +116,8 @@ class EnumSerializer(BaseSerializer):
     def serialize(self):
         enum_class = self.value.__class__
         module = enum_class.__module__
-        imports = {"import %s" % module}
         v_string, v_imports = serializer_factory(self.value.value).serialize()
-        imports.update(v_imports)
+        imports = {'import %s' % module, *v_imports}
         return "%s.%s(%s)" % (module, enum_class.__name__, v_string), imports
 
 
@@ -166,18 +157,18 @@ class FunctionTypeSerializer(BaseSerializer):
 
 class FunctoolsPartialSerializer(BaseSerializer):
     def serialize(self):
-        imports = {'import functools'}
         # Serialize functools.partial() arguments
         func_string, func_imports = serializer_factory(self.value.func).serialize()
         args_string, args_imports = serializer_factory(self.value.args).serialize()
         keywords_string, keywords_imports = serializer_factory(self.value.keywords).serialize()
         # Add any imports needed by arguments
-        imports.update(func_imports)
-        imports.update(args_imports)
-        imports.update(keywords_imports)
+        imports = {'import functools', *func_imports, *args_imports, *keywords_imports}
         return (
-            "functools.partial(%s, *%s, **%s)" % (
-                func_string, args_string, keywords_string,
+            'functools.%s(%s, *%s, **%s)' % (
+                self.value.__class__.__name__,
+                func_string,
+                args_string,
+                keywords_string,
             ),
             imports,
         )
@@ -223,14 +214,12 @@ class OperationSerializer(BaseSerializer):
 
 class RegexSerializer(BaseSerializer):
     def serialize(self):
-        imports = {"import re"}
         regex_pattern, pattern_imports = serializer_factory(self.value.pattern).serialize()
         # Turn off default implicit flags (e.g. re.U) because regexes with the
         # same implicit and explicit flags aren't equal.
         flags = self.value.flags ^ re.compile('').flags
         regex_flags, flag_imports = serializer_factory(flags).serialize()
-        imports.update(pattern_imports)
-        imports.update(flag_imports)
+        imports = {'import re', *pattern_imports, *flag_imports}
         args = [regex_pattern]
         if flags:
             args.append(regex_flags)
@@ -254,24 +243,6 @@ class SettingsReferenceSerializer(BaseSerializer):
         return "settings.%s" % self.value.setting_name, {"from django.conf import settings"}
 
 
-class TextTypeSerializer(BaseSerializer):
-    def serialize(self):
-        return repr(self.value), set()
-
-
-class TimedeltaSerializer(BaseSerializer):
-    def serialize(self):
-        return repr(self.value), {"import datetime"}
-
-
-class TimeSerializer(BaseSerializer):
-    def serialize(self):
-        value_repr = repr(self.value)
-        if isinstance(self.value, datetime_safe.time):
-            value_repr = "datetime.%s" % value_repr
-        return value_repr, {"import datetime"}
-
-
 class TupleSerializer(BaseSequenceSerializer):
     def _format(self):
         # When len(value)==0, the empty tuple should be serialized as "()",
@@ -283,6 +254,7 @@ class TypeSerializer(BaseSerializer):
     def serialize(self):
         special_cases = [
             (models.Model, "models.Model", []),
+            (type(None), 'type(None)', []),
         ]
         for case, string, imports in special_cases:
             if case is self.value:
@@ -300,8 +272,39 @@ class UUIDSerializer(BaseSerializer):
         return "uuid.%s" % repr(self.value), {"import uuid"}
 
 
+class Serializer:
+    _registry = OrderedDict([
+        (frozenset, FrozensetSerializer),
+        (list, SequenceSerializer),
+        (set, SetSerializer),
+        (tuple, TupleSerializer),
+        (dict, DictionarySerializer),
+        (enum.Enum, EnumSerializer),
+        (datetime.datetime, DatetimeDatetimeSerializer),
+        ((datetime.date, datetime.timedelta, datetime.time), DateTimeSerializer),
+        (SettingsReference, SettingsReferenceSerializer),
+        (float, FloatSerializer),
+        ((bool, int, type(None), bytes, str), BaseSimpleSerializer),
+        (decimal.Decimal, DecimalSerializer),
+        ((functools.partial, functools.partialmethod), FunctoolsPartialSerializer),
+        ((types.FunctionType, types.BuiltinFunctionType, types.MethodType), FunctionTypeSerializer),
+        (collections.abc.Iterable, IterableSerializer),
+        ((COMPILED_REGEX_TYPE, RegexObject), RegexSerializer),
+        (uuid.UUID, UUIDSerializer),
+    ])
+
+    @classmethod
+    def register(cls, type_, serializer):
+        if not issubclass(serializer, BaseSerializer):
+            raise ValueError("'%s' must inherit from 'BaseSerializer'." % serializer.__name__)
+        cls._registry[type_] = serializer
+
+    @classmethod
+    def unregister(cls, type_):
+        cls._registry.pop(type_)
+
+
 def serializer_factory(value):
-    from django.db.migrations.writer import SettingsReference
     if isinstance(value, Promise):
         value = str(value)
     elif isinstance(value, LazyObject):
@@ -320,50 +323,9 @@ def serializer_factory(value):
     # Anything that knows how to deconstruct itself.
     if hasattr(value, 'deconstruct'):
         return DeconstructableSerializer(value)
-
-    # Unfortunately some of these are order-dependent.
-    if isinstance(value, frozenset):
-        return FrozensetSerializer(value)
-    if isinstance(value, list):
-        return SequenceSerializer(value)
-    if isinstance(value, set):
-        return SetSerializer(value)
-    if isinstance(value, tuple):
-        return TupleSerializer(value)
-    if isinstance(value, dict):
-        return DictionarySerializer(value)
-    if isinstance(value, enum.Enum):
-        return EnumSerializer(value)
-    if isinstance(value, datetime.datetime):
-        return DatetimeSerializer(value)
-    if isinstance(value, datetime.date):
-        return DateSerializer(value)
-    if isinstance(value, datetime.time):
-        return TimeSerializer(value)
-    if isinstance(value, datetime.timedelta):
-        return TimedeltaSerializer(value)
-    if isinstance(value, SettingsReference):
-        return SettingsReferenceSerializer(value)
-    if isinstance(value, float):
-        return FloatSerializer(value)
-    if isinstance(value, (bool, int, type(None))):
-        return BaseSimpleSerializer(value)
-    if isinstance(value, bytes):
-        return ByteTypeSerializer(value)
-    if isinstance(value, str):
-        return TextTypeSerializer(value)
-    if isinstance(value, decimal.Decimal):
-        return DecimalSerializer(value)
-    if isinstance(value, functools.partial):
-        return FunctoolsPartialSerializer(value)
-    if isinstance(value, (types.FunctionType, types.BuiltinFunctionType, types.MethodType)):
-        return FunctionTypeSerializer(value)
-    if isinstance(value, collections.Iterable):
-        return IterableSerializer(value)
-    if isinstance(value, (COMPILED_REGEX_TYPE, RegexObject)):
-        return RegexSerializer(value)
-    if isinstance(value, uuid.UUID):
-        return UUIDSerializer(value)
+    for type_, serializer_cls in Serializer._registry.items():
+        if isinstance(value, type_):
+            return serializer_cls(value)
     raise ValueError(
         "Cannot serialize: %r\nThere are some values Django cannot serialize into "
         "migration files.\nFor more, see https://docs.djangoproject.com/en/%s/"
