@@ -99,6 +99,12 @@ class BaseForm:
         if use_required_attribute is not None:
             self.use_required_attribute = use_required_attribute
 
+        if self.empty_permitted and self.use_required_attribute:
+            raise ValueError(
+                'The empty_permitted and use_required_attribute arguments may '
+                'not both be True.'
+            )
+
         # Initialize form renderer. Use a global default if not specified
         # either as an argument or as self.default_renderer.
         if renderer is None:
@@ -139,7 +145,7 @@ class BaseForm:
         if self._errors is None:
             is_valid = "Unknown"
         else:
-            is_valid = self.is_bound and not bool(self._errors)
+            is_valid = self.is_bound and not self._errors
         return '<%(cls)s bound=%(bound)s, valid=%(valid)s, fields=(%(fields)s)>' % {
             'cls': self.__class__.__name__,
             'bound': self.is_bound,
@@ -199,8 +205,7 @@ class BaseForm:
         for name, field in self.fields.items():
             html_class_attr = ''
             bf = self[name]
-            # Escape and cache in local variable.
-            bf_errors = self.error_class([conditional_escape(error) for error in bf.errors])
+            bf_errors = self.error_class(bf.errors)
             if bf.is_hidden:
                 if bf_errors:
                     top_errors.extend(
@@ -275,8 +280,9 @@ class BaseForm:
             normal_row='<tr%(html_class_attr)s><th>%(label)s</th><td>%(errors)s%(field)s%(help_text)s</td></tr>',
             error_row='<tr><td colspan="2">%s</td></tr>',
             row_ender='</td></tr>',
-            help_text_html='<br /><span class="helptext">%s</span>',
-            errors_on_separate_row=False)
+            help_text_html='<br><span class="helptext">%s</span>',
+            errors_on_separate_row=False,
+        )
 
     def as_ul(self):
         "Return this form rendered as HTML <li>s -- excluding the <ul></ul>."
@@ -285,7 +291,8 @@ class BaseForm:
             error_row='<li>%s</li>',
             row_ender='</li>',
             help_text_html=' <span class="helptext">%s</span>',
-            errors_on_separate_row=False)
+            errors_on_separate_row=False,
+        )
 
     def as_p(self):
         "Return this form rendered as HTML <p>s."
@@ -294,7 +301,8 @@ class BaseForm:
             error_row='%s',
             row_ender='</p>',
             help_text_html=' <span class="helptext">%s</span>',
-            errors_on_separate_row=True)
+            errors_on_separate_row=True,
+        )
 
     def non_field_errors(self):
         """
@@ -352,13 +360,10 @@ class BaseForm:
                 del self.cleaned_data[field]
 
     def has_error(self, field, code=None):
-        if code is None:
-            return field in self.errors
-        if field in self.errors:
-            for error in self.errors.as_data()[field]:
-                if error.code == code:
-                    return True
-        return False
+        return field in self.errors and (
+            code is None or
+            any(error.code == code for error in self.errors.as_data()[field])
+        )
 
     def full_clean(self):
         """
@@ -465,10 +470,7 @@ class BaseForm:
         Return True if the form needs to be multipart-encoded, i.e. it has
         FileInput, or False otherwise.
         """
-        for field in self.fields.values():
-            if field.widget.needs_multipart_form:
-                return True
-        return False
+        return any(field.widget.needs_multipart_form for field in self.fields.values())
 
     def hidden_fields(self):
         """
