@@ -1,9 +1,6 @@
-import logging
 import posixpath
-import warnings
 from collections import defaultdict
 
-from django.utils.deprecation import RemovedInDjango21Warning
 from django.utils.safestring import mark_safe
 
 from .base import (
@@ -15,8 +12,6 @@ register = Library()
 
 BLOCK_CONTEXT_KEY = 'block_context'
 
-logger = logging.getLogger('django.template')
-
 
 class BlockContext:
     def __init__(self):
@@ -170,46 +165,27 @@ class IncludeNode(Node):
         in render_context to avoid reparsing and loading when used in a for
         loop.
         """
-        try:
-            template = self.template.resolve(context)
-            # Does this quack like a Template?
-            if not callable(getattr(template, 'render', None)):
-                # If not, we'll try our cache, and get_template()
-                template_name = template
-                cache = context.render_context.dicts[0].setdefault(self, {})
-                template = cache.get(template_name)
-                if template is None:
-                    template = context.template.engine.get_template(template_name)
-                    cache[template_name] = template
-            # Use the base.Template of a backends.django.Template.
-            elif hasattr(template, 'template'):
-                template = template.template
-            values = {
-                name: var.resolve(context)
-                for name, var in self.extra_context.items()
-            }
-            if self.isolated_context:
-                return template.render(context.new(values))
-            with context.push(**values):
-                return template.render(context)
-        except Exception as e:
-            if context.template.engine.debug:
-                raise
-            template_name = getattr(context, 'template_name', None) or 'unknown'
-            warnings.warn(
-                "Rendering {%% include '%s' %%} raised %s. In Django 2.1, "
-                "this exception will be raised rather than silenced and "
-                "rendered as an empty string." %
-                (template_name, e.__class__.__name__),
-                RemovedInDjango21Warning,
-            )
-            logger.warning(
-                "Exception raised while rendering {%% include %%} for "
-                "template '%s'. Empty string rendered instead.",
-                template_name,
-                exc_info=True,
-            )
-            return ''
+        template = self.template.resolve(context)
+        # Does this quack like a Template?
+        if not callable(getattr(template, 'render', None)):
+            # If not, try the cache and get_template().
+            template_name = template
+            cache = context.render_context.dicts[0].setdefault(self, {})
+            template = cache.get(template_name)
+            if template is None:
+                template = context.template.engine.get_template(template_name)
+                cache[template_name] = template
+        # Use the base.Template of a backends.django.Template.
+        elif hasattr(template, 'template'):
+            template = template.template
+        values = {
+            name: var.resolve(context)
+            for name, var in self.extra_context.items()
+        }
+        if self.isolated_context:
+            return template.render(context.new(values))
+        with context.push(**values):
+            return template.render(context)
 
 
 @register.tag('block')
