@@ -179,7 +179,8 @@ def number_format(value, decimal_pos=None, use_l10n=None, force_grouping=False):
         decimal_pos,
         get_format('NUMBER_GROUPING', lang, use_l10n=use_l10n),
         get_format('THOUSAND_SEPARATOR', lang, use_l10n=use_l10n),
-        force_grouping=force_grouping
+        force_grouping=force_grouping,
+        use_l10n=use_l10n,
     )
 
 
